package devoir03.modele;

import java.io.*;

/**
 * Classe d'entrées/sortie : lit les documents textes (.txt) dans le dossier
 * passé en argument au constructeur. Les fichiers sont supposés être dans le
 * codage du système (vous trouverez avec le projet un ensemble de fichiers
 * codés en ASCII, qui passent partout).
 * <p>
 * ATTENTION: pour ce devoir, on demande explicitement de stocker les documents
 * en mémoire.
 * <p>
 * Le lecteur de document ne doit être utilisé qu'une seule fois.
 * 
 * <p>
 * Boucle de lecture :
 * 
 * <pre>
 * LecteurDeDocuments l = new LecteurDeDocuments(&quot;monDossier&quot;);
 * while (l.aDocument()) {
 * 	String nom = l.getNomFichier();
 * 	String contenu = l.getContenuFichier();
 * ... votre code...
 * 	l.suivant();
 * }
 * </pre>
 * 
 * @author rosmord
 */
public class LecteursDeDocuments {

	File[] documents;
	String contenuDocumentCourant;
	/**
	 * Position du document courant dans la liste. -1 : pas encore sur un
	 * document.
	 */
	int position;

	/**
	 * Construit un lecteur de documents qui va récupérer les documents textes
	 * dans le dossier <code>dossier</dossier>
	 * 
	 * @param dossier
	 *            le dossier où se trouvent les documents.
	 */
	public LecteursDeDocuments(File dossier) {
		documents = dossier.listFiles(new FiltreTxt());
		position = 0;
		lirePremierDocumentDisponible();
	}

	/**
	 * Renvoie vrai s'il y a un document à consulter.
	 * 
	 * @return un booléen.
	 */
	public boolean aDocument() {
		return 0 <= position && position < documents.length;
	}

	/**
	 * Avance au prochain document.
	 * <p>
	 * Si on était sur le dernier document, cette méthode n'échoue pas, mais,
	 * par la suite, {@link #aDocument()} renverra false.
	 * <p>
	 * Postcondition : on est sur un nouveau document, ou {@link #aDocument()}
	 * vaut false.
	 */

	public void suivant() {
		position++;
		lirePremierDocumentDisponible();
	}

	/**
	 * Se place, si possible, sur le premier document lisible à partir de la
	 * position courante.
	 */
	private void lirePremierDocumentDisponible() {
		boolean aFaire = true;
		// On va avancer jusqu'à trouver un document qui soit lisible, ou à être
		// sorti
		// de la liste des documents.
		while (aFaire && position < documents.length) {
			// Si on est sur un document, on essaie de le lire:
			try {
				// Le lecteur (on lit ligne par ligne, 
				// en ajoutant le changement de lignes "unix").
				BufferedReader reader = new BufferedReader(new FileReader(
						documents[position]));
				// Pour concaténer efficacement des strings on utilise
				// des StringBuilder.
				// Je voulais faire du code simple et lisible,
				// mais avec de la concaténation de Strings normale,
				// la lecture du fichier d'exemple prend 5 secondes.
				//
				StringBuilder contenu = new StringBuilder();
				String ligne= reader.readLine();
				while (ligne != null) {
					contenu.append(ligne + "\n");
					ligne= reader.readLine();
				}
				// close se fait normalement de manière plus parano,
				// mais je reste simple.
				reader.close();
				contenuDocumentCourant = contenu.toString();
				// Si on arrive ici, on a le contenu,
				// on a lu un document, et on s'arrête.
				aFaire = false;
			} catch (IOException e) {
				// On a échoué (par exemple ce document n'est pas lisible - ce
				// qui ne devrait pas arriver dans notre application).
				// On essaie le suivant:
				position++;
			}
		}
	}

	/**
	 * Retourne le nom du document courant.
	 * <p>
	 * Précondition : {@link #aDocument()} == true.
	 * 
	 * @return le document courant
	 * @throws IllegalStateException
	 *             si la précondition n'est pas respectée.
	 */
	public String getNomFichier() {
		if (!aDocument())
			throw new IllegalStateException("plus de documents");
		return documents[position].getName();
	}

	public String getContenuFichier() {
		if (!aDocument())
			throw new IllegalStateException("plus de documents");
		return contenuDocumentCourant;
	}

	/**
	 * Filtre utilisé pour ne lire que les documents textes.
	 * 
	 * @author rosmord
	 * 
	 */
	private static class FiltreTxt implements FilenameFilter {
		@Override
		public boolean accept(File file, String name) {
			File subFile = new File(file, name);
			return name.endsWith(".txt") && subFile.canRead()
					&& subFile.isFile();
		}
	}

}
